package cn.iocoder.yudao.module.system.controller.admin.equipment;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.attendancemachine.AttendanceMachinePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.attendancemachine.AttendanceMachineRespVO;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.attendancemachine.AttendanceMachineSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.equipment.AttendanceMachineDO;
import cn.iocoder.yudao.module.system.service.equipment.AttendanceMachineService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;


@Tag(name = "管理后台 - 考勤设备")
@RestController
@RequestMapping("/system/attendance-machine")
@Validated
public class AttendanceMachineController {

    @Resource
    private AttendanceMachineService attendanceMachineService;

    @PostMapping("/create")
    @Operation(summary = "创建考勤设备")
    @PreAuthorize("@ss.hasPermission('system:attendance-machine:create')")
    public CommonResult<Long> createAttendanceMachine(@Valid @RequestBody AttendanceMachineSaveReqVO createReqVO) {
        return success(attendanceMachineService.createAttendanceMachine(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新考勤设备")
    @PreAuthorize("@ss.hasPermission('system:attendance-machine:update')")
    public CommonResult<Boolean> updateAttendanceMachine(@Valid @RequestBody AttendanceMachineSaveReqVO updateReqVO) {
        attendanceMachineService.updateAttendanceMachine(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除考勤设备")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('system:attendance-machine:delete')")
    public CommonResult<Boolean> deleteAttendanceMachine(@RequestParam("id") Long id) {
        attendanceMachineService.deleteAttendanceMachine(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得考勤设备")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('system:attendance-machine:query')")
    public CommonResult<AttendanceMachineRespVO> getAttendanceMachine(@RequestParam("id") Long id) {
        AttendanceMachineDO attendanceMachine = attendanceMachineService.getAttendanceMachine(id);
        return success(BeanUtils.toBean(attendanceMachine, AttendanceMachineRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得考勤设备分页")
    @PreAuthorize("@ss.hasPermission('system:attendance-machine:query')")
    public CommonResult<PageResult<AttendanceMachineRespVO>> getAttendanceMachinePage(@Valid AttendanceMachinePageReqVO pageReqVO) {
        PageResult<AttendanceMachineDO> pageResult = attendanceMachineService.getAttendanceMachinePage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, AttendanceMachineRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出考勤设备 Excel")
    @PreAuthorize("@ss.hasPermission('system:attendance-machine:export')")
    @OperateLog(type = EXPORT)
    public void exportAttendanceMachineExcel(@Valid AttendanceMachinePageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<AttendanceMachineDO> list = attendanceMachineService.getAttendanceMachinePage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "考勤设备.xls", "数据", AttendanceMachineRespVO.class,
                        BeanUtils.toBean(list, AttendanceMachineRespVO.class));
    }

}