package cn.iocoder.yudao.module.system.controller.admin.equipment;

import cn.hutool.core.collection.CollUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.collection.CollectionUtils;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.attendancemachine.AttendanceMachineRespVO;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.userExt.UsersExtPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.userExt.UsersExtRespVO;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.userExt.UsersExtSaveReqVO;
import cn.iocoder.yudao.module.system.convert.equipment.UsersExtConvert;
import cn.iocoder.yudao.module.system.dal.dataobject.dept.DeptDO;
import cn.iocoder.yudao.module.system.dal.dataobject.equipment.AttendanceMachineDO;
import cn.iocoder.yudao.module.system.dal.dataobject.equipment.UsersExtDO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.service.dept.DeptService;
import cn.iocoder.yudao.module.system.service.equipment.AttendanceMachineService;
import cn.iocoder.yudao.module.system.service.equipment.UsersExtService;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.common.util.collection.CollectionUtils.convertList;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;



@Tag(name = "管理后台 - 用户信息拓展")
@RestController
@RequestMapping("/system/users-ext")
@Validated
public class UsersExtController {

    @Resource
    private UsersExtService usersExtService;

    @Resource
    private AdminUserService userService;

    @Resource
    private DeptService deptService;

    @Resource
    private AttendanceMachineService attendanceMachineService;

    @PostMapping("/create")
    @Operation(summary = "上传照片")
    @PreAuthorize("@ss.hasPermission('system:users-ext:create')")
    public CommonResult<Long> createUsersExt(@Valid @RequestBody UsersExtSaveReqVO createReqVO) {
        return success(usersExtService.createUsersExt(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "删除用户已下发的设备")
    @PreAuthorize("@ss.hasPermission('system:users-ext:update')")
    public CommonResult<Boolean> updateUsersExt(@Valid @RequestBody UsersExtSaveReqVO updateReqVO) {
        usersExtService.updateUsersExt(updateReqVO);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得用户下发设备详情")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('system:users-ext:query')")
    public CommonResult<List<AttendanceMachineRespVO>> getAttendanceMachine(@RequestParam("id") Long id) {

        UsersExtDO usersExt = usersExtService.getUsersExt(id);

        // 获得用户已下发的设备列表
        List<AttendanceMachineDO> attendanceMachineDO = attendanceMachineService.getAttendanceMachineByDeviceNo(usersExt.getAttendanceMachineNos());

        // 获得部门信息Map
        Map<Long, DeptDO> deptDOMap = deptService.getDeptMap(convertList(attendanceMachineDO, AttendanceMachineDO::getDeptId));

        List<AttendanceMachineRespVO> respVOS = BeanUtils.toBean(attendanceMachineDO, AttendanceMachineRespVO.class);

        return success(CollectionUtils.convertList(respVOS, respVO -> respVO.setDeptName(deptDOMap.get(respVO.getDeptId()).getName())));
    }

    @GetMapping("/page")
    @Operation(summary = "获得用户信息拓展分页")
    @PreAuthorize("@ss.hasPermission('system:users-ext:query')")
    public CommonResult<PageResult<UsersExtRespVO>> getUsersExtPage(@Valid UsersExtPageReqVO pageReqVO) {

        PageResult<UsersExtDO> pageResult = usersExtService.getUsersExtPage(pageReqVO);
        if (!CollUtil.isEmpty(pageResult.getList())) {

            //获得用户Map
            Map<Long, AdminUserDO> userMap = userService.getUserMap(convertList(pageResult.getList(), UsersExtDO::getUserId));
            //获得部门Map
            Map<Long, DeptDO> deptMap = deptService.getDeptMap(convertList(pageResult.getList(), UsersExtDO::getDeptId));

            return success(new PageResult<>(UsersExtConvert.INSTANCE.convertList(pageResult.getList(), userMap, deptMap)
                    , pageResult.getTotal()));
        }

        return success(BeanUtils.toBean(pageResult, UsersExtRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出用户信息拓展 Excel")
    @PreAuthorize("@ss.hasPermission('system:users-ext:export')")
    @OperateLog(type = EXPORT)
    public void exportUsersExtExcel(@Valid UsersExtPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<UsersExtDO> list = usersExtService.getUsersExtPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "用户信息拓展.xls", "数据", UsersExtRespVO.class,
                        BeanUtils.toBean(list, UsersExtRespVO.class));
    }

}