package cn.iocoder.yudao.module.system.service.equipment;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.attendancemachine.AttendanceMachinePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.attendancemachine.AttendanceMachineSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.equipment.AttendanceMachineDO;
import cn.iocoder.yudao.module.system.dal.mysql.equipment.AttendanceMachineMapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;

import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.ATTENDANCE_MACHINE_NOT_EXISTS;

/**
 * 考勤设备 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class AttendanceMachineServiceImpl implements AttendanceMachineService {

    @Resource
    private AttendanceMachineMapper attendanceMachineMapper;

    @Override
    public Long createAttendanceMachine(AttendanceMachineSaveReqVO createReqVO) {
        // 插入
        AttendanceMachineDO attendanceMachine = BeanUtils.toBean(createReqVO, AttendanceMachineDO.class);
        attendanceMachineMapper.insert(attendanceMachine);
        // 返回
        return attendanceMachine.getId();
    }

    @Override
    public void updateAttendanceMachine(AttendanceMachineSaveReqVO updateReqVO) {
        // 校验存在
        validateAttendanceMachineExists(updateReqVO.getId());
        // 更新
        AttendanceMachineDO updateObj = BeanUtils.toBean(updateReqVO, AttendanceMachineDO.class);
        attendanceMachineMapper.updateById(updateObj);
    }

    @Override
    public void deleteAttendanceMachine(Long id) {
        // 校验存在
        validateAttendanceMachineExists(id);
        // 删除
        attendanceMachineMapper.deleteById(id);
    }

    private void validateAttendanceMachineExists(Long id) {
        if (attendanceMachineMapper.selectById(id) == null) {
            throw exception(ATTENDANCE_MACHINE_NOT_EXISTS);
        }
    }

    @Override
    public AttendanceMachineDO getAttendanceMachine(Long id) {

        return attendanceMachineMapper.selectById(id);
    }

    @Override
    public List<AttendanceMachineDO> getAttendanceMachineByDeviceNo(List<String> deviceNo) {

        return attendanceMachineMapper.selectList(AttendanceMachineDO::getDeviceNo, deviceNo);
    }

    @Override
    public PageResult<AttendanceMachineDO> getAttendanceMachinePage(AttendanceMachinePageReqVO pageReqVO) {

        return attendanceMachineMapper.selectPage(pageReqVO);
    }

}