package cn.iocoder.yudao.module.system.service.equipment;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.userExt.UsersExtPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.userExt.UsersExtSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.equipment.UsersExtDO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.dal.mysql.equipment.UsersExtMapper;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Collections;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.util.collection.CollectionUtils.convertList;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.USERS_EXT_NOT_EXISTS;

/**
 * 用户信息拓展 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class UsersExtServiceImpl implements UsersExtService {

    @Resource
    private UsersExtMapper usersExtMapper;

    @Resource
    @Lazy
    private AdminUserService userService;

    @Override
    public Long createUsersExt(UsersExtSaveReqVO createReqVO) {

        // 插入
        UsersExtDO usersExt = BeanUtils.toBean(createReqVO, UsersExtDO.class);
        usersExtMapper.insert(usersExt);
        // 返回
        return usersExt.getId();
    }

    @Override
    public void updateUsersExt(UsersExtSaveReqVO updateReqVO) {
        // 校验存在
        validateUsersExtExists(updateReqVO.getUserId());
        // 更新
        UsersExtDO updateObj = BeanUtils.toBean(updateReqVO, UsersExtDO.class);

        usersExtMapper.updateById(updateObj);
    }

    private void validateUsersExtExists(Long userId) {

        if (usersExtMapper.selectOne(UsersExtDO::getUserId, userId) == null) {
            throw exception(USERS_EXT_NOT_EXISTS);
        }
    }

    @Override
    public UsersExtDO getUsersExt(Long id) {
        return usersExtMapper.selectById(id);
    }

    @Override
    public PageResult<UsersExtDO> getUsersExtPage(UsersExtPageReqVO pageReqVO) {

        if (pageReqVO.getUserName() != null) {

            pageReqVO.setUserIds(convertList(userService.getUserListByNickname(pageReqVO.getUserName()), AdminUserDO::getId));
        }

        if (pageReqVO.getPhoneNumber() != null) {

            AdminUserDO userDO = userService.getUserByMobile(pageReqVO.getPhoneNumber());
            if (userDO != null) {

                pageReqVO.setUserIds(Collections.singletonList(userDO.getId()));
            }
        }

        return usersExtMapper.selectPage(pageReqVO);
    }
}